
; small probe										.
;				____	____						.
;		   +5V |	\__/	| GND					.
;	LED RED A5 |			| A0 PGD PROBE OUT		.
;   LED YEL A4 |   12F1572  | A1 PGC PROBE IN		.
;     Vpp MCLR |____________| A2 LED GREEN			.

	list	p=12f1572
	#include <p12f1572.inc>
	#include <macros.inc>
	errorlevel -302

 __CONFIG _CONFIG1,_FOSC_INTOSC&_WDTE_NSLEEP&_PWRTE_ON&_MCLRE_ON&_CP_OFF&_BOREN_OFF&_CLKOUTEN_OFF
 __CONFIG _CONFIG2,_WRT_OFF&_PLLEN_ON&_STVREN_OFF&_BORV_19&_LPBOREN_OFF&_LVP_OFF
;																													.

	CBLOCK	70h		; Uses only 16-byte common RAM

countdown	; for PULSE LED timing
uniform		; to slow down PULSE LED timing
outbyte		; temporary PORTA state, before displaying out
inner		; simple timing loop
prev		; previous state of OUTBYTE, for single transition detection
off_lo		; auto power off low counter
off_next	; auto power off next counter
off_hi		; auto power off high counter

	ENDC
;																													.
;SAMPLE1		EQU	0x7D	;
	org		0
start
; ***********************   Start initialization
	clrf	INTCON		; disable interrupts
  bank0
	clrf	T1CON		; stop TMR1
	clrf	T2CON		; stop TMR2
  bank1
	movlw	b'001010'	; portA dir
	movwf	TRISA
	movlw	b'10001000'
	movwf	OPTION_REG	; disable pullups
	movlw	b'11111010'
	movwf	OSCCON		; Internal osc 16 MHz
	clrf	PIE1
  bank2
	clrf	LATA		; turn off LEDS
  bank3
	clrf	ANSELA		; portA = digital I/O
  bank6
	movlw	b'111100'
	movwf	SLRCONA		; limiter slew rate on LED pins, fast on DACOUT pin
  bank7
	movlw	b'00000010'	; schmitt trigger input portA1
	movwf	INLVLA		

	movlw	b'00000010'
	movwf	IOCAP		; interrupt-on-change positive edge enable
	movwf	IOCAN		; interrupt-on-change negative edge enable
; ***********************   End initialization
	clrf	FSR0H
	movlw	0x70
	movwf	FSR0L
clram
	clrf	INDF0		; clr internal RAM from 70h to 7fh
	incf	FSR0L,f
	btfss	FSR0L,7
	goto	clram

	movlw	.69			; 69 = 10 minutes
	movwf	off_hi		; auto power off timing (8.65 s for 65536 loop)

	call	opening
; ***********************   Start main pgm loop
farm
;					GREEN LED SERVICE (low)  G G G G G G G G G G G G G G G G G G
  bank7
		clrf	IOCAP		; *** hole start (positive going transitions will me missed)
  bank2
		bsf		LATA,0		; ___/^^^  hi-imp probe output = HIGH
		bsf		outbyte,0
		call	wait		; 24t
  bank7
		bsf		IOCAP,1		; *** hole end
;	...
  bank0
	movf	outbyte,w
	movwf	prev
	bcf		outbyte,2		; default GREEN LED OFF
	btfss	PORTA,1			; if input lo, then...
	bsf		outbyte,2		; ...GREEN LED ON
	movf	outbyte,w
	movwf	PORTA
;	...
	call	yellow_service	; 224t
;					RED LED SERVICE (high)   R R R R R R R R R R R R R R R R R R 
  bank7
		clrf	IOCAN		; *** hole start (negative going transitions will me missed)
  bank2
		bcf		LATA,0		; ^^^\___  hi-imp probe output = LO
		bcf		outbyte,0
		call	wait		; 24t
  bank7
		bsf		IOCAN,1		; *** hole end
;	...
  bank0
	movf	outbyte,w
	movwf	prev
	bcf		outbyte,5		; default RED LED OFF
	btfsc	PORTA,1			; if input hi, then...
	bsf		outbyte,5		; ...RED LED ON
	movf	outbyte,w
	movwf	PORTA
;	...
	call	yellow_service	; 224t
;					POWER OFF SERVICE											.
	decfsz	off_lo,f
	bra		farm			; 528t here
	decfsz	off_next,f
	bra		farm
	decfsz	off_hi,f
	bra		farm
					; time to switch off
  bank2
	call	closing
	clrf	LATA		; LEDs off
  bank7
	clrf	IOCAP		; interrupt-on-change positive edge disable
	clrf	IOCAN		; interrupt-on-change negative edge disable
	clrf	IOCAF		; clear interrupt-on-change flags
  bank1
	clrf	TRISA		; All pins outputs, to protect input
	movlw	b'10001000'
	movwf	OPTION_REG	; disable pullups
	sleep				; THE ONLY EXIT IS RESET, BUT...
	goto	start		; ...JUST IN CASE
; ***********************   End main pgm loop    528t = 7.57KHz (132s) @16MHz

; ***********************   Start subroutines
;					YELLOW LED SERVICE (pulse)   Y Y Y Y Y Y Y Y Y Y Y Y Y Y Y Y 
yellow_service
	clrwdt

	movf	prev,w
	xorwf	outbyte,w
	andlw	b'00100100'
	ifnz
	bra		yes_pulse		; single transition detected by software
;	...
  bank7
	btfss	IOCAF,1
	bra		no_pulse
yes_pulse					; transition detected by Interrupt On Change
		movlw	0xFF
		xorwf	IOCAF,w
		andwf	IOCAF,f			; clear interrupt flag

		bsf		outbyte,4		; YEL LED ON
		movlw	.255
		movwf	countdown

;		movlw	.69				; 69 = 10 minutes auto power off timing
;		movwf	off_hi
		bra		exit
;	...
no_pulse					; transition NOT detected
		movf	countdown,f		; dummy, test for Z
		ifz
		bcf		outbyte,4		; if countdown Z, YEL LED OFF
		ifz
		bra		exit
		incf	uniform,f
		movf	uniform,w
		andlw	b'11'			; adjust LED PULSE period here
		ifz
		decf	countdown,f		; if countdown NZ, then decrement countdown
exit
  bank2
	movf	outbyte,w
	movwf	LATA
	movlw	.50
	movwf	inner
loop
	nop					; 50t
	decfsz	inner,f		; 50t
	bra		loop		; 99t

	return
;								.
wait
    variable zz
zz=0
    while zz<.20
	nop
zz+=.1
    endw
	return
;								.
opening
  bank2
	bsf		LATA,2			; green on
	call	ani_pause
	bsf		LATA,5			; red on
	call	ani_pause
	bsf		LATA,4			; yellow on
	call	ani_pause
	bcf		LATA,2			; green off
	call	ani_pause
	bcf		LATA,5			; red off
	call	ani_pause
	bcf		LATA,4			; yellow off
	return
;	.
closing
  bank2
	bsf		LATA,4			; yellow on
	call	ani_pause
	bsf		LATA,5			; red on
	call	ani_pause
	bsf		LATA,2			; green on
	call	ani_pause
	bcf		LATA,4			; yellow off
	call	ani_pause
	bcf		LATA,5			; red off
	call	ani_pause
	bcf		LATA,2			; green off
	return
;	.
ani_pause
	clrf	uniform
	clrf	inner
go_pause
	nop
	decfsz	inner,f
	bra		go_pause
	decfsz	uniform,f
	bra		go_pause
	return

; ***********************   End subroutines

	end

